(* ::Package:: *)

(* ::Title::Closed:: *)
(*Quest+*)


(* ::SmallText:: *)
(*\[Copyright] 2017 Andrew B. Watson*)


(* ::Section::Closed:: *)
(*Introduction*)


(* ::Text:: *)
(*This Notebook contains an implementation and demonstrations of the QUEST+  general multidimensional Bayesian adaptive psychometric method. The method is described in the publication below.*)


(* ::Text:: *)
(*Watson, A. B. (2017). QUEST+: a general multidimensional Bayesian adaptive psychometric method. Journal of Vision.*)


(* ::Section::Closed:: *)
(*Credits & Version*)


(* ::Text:: *)
(*Copyright 2017 by Andrew B. Watson. All rights reserved. Do not distribute or reproduce without permission. No warranties expressed or implied. This Version is compatible with Mathematica 11.*)


(* ::Input:: *)
(*DateString[]*)


(* ::Section::Closed:: *)
(*Styles*)


(* ::Input::Initialization:: *)
QpStyles = {Frame->True,Axes->False,GridLines->Automatic,GridLinesStyle->Directive[GrayLevel[0.85],Thickness[Tiny]],BaseStyle->{12,FontFamily->"Helvetica",AbsolutePointSize[6]},PlotRange->All,ImageSize->{Automatic,240}};


(* ::Input::Initialization:: *)
QpFigureStyle={LabelStyle->{16,FontFamily->"Helvetica"},BaseStyle->{16,FontFamily->"Helvetica"},FrameTicksStyle->16,Frame->True,ImageSize->{Automatic,300}};


(* ::Input::Initialization:: *)
DemoStyle={
LabelStyle->DemoControlLabelStyle
,BaseStyle->{14,FontFamily->"Helvetica"}
,Paneled->False
,SaveDefinitions->True
,ControlPlacement->Bottom
,ContentSize->Automatic
,Alignment->Center
};


(* ::Input::Initialization:: *)
DemoTitleStyle={32,FontColor->RGBColor[0,150/255,1],FontWeight->"Light",FontFamily->"Helvetica-Neue"};
DemoReferenceStyle={10,FontColor->Black,FontFamily->"Helvetica-Neue"};
DemoNotesStyle=DemoReferenceStyle;
DemoIntroStyle = {14,FontColor->Black,FontFamily->"Helvetica-Neue"};
DemoInstructionsStyle={14,FontColor->Black,FontFamily->"Helvetica-Neue"};
DemoItemSize=Automatic;
DemoControlLabelStyle={18,FontColor->Gray,FontFamily->"Helvetica-Neue"};


(* ::Input::Initialization:: *)
DateTimeString:=DateString[{"MonthNameShort"," ", "Day"," ","Year"," ","Time"," UTC"},TimeZone->0]


(* ::Input::Initialization:: *)
DemoFrameLabels[title_,intro_,instructions_,notes_,references_] := {Column[{Spacer[{1,2}]
,Style[intro,DemoIntroStyle]
,Style[instructions,DemoInstructionsStyle]
,Style[notes,DemoNotesStyle]
,Style[references,DemoReferenceStyle]
}
,Alignment->Left,ItemSize->DemoItemSize]
,None
,Column[{Style[title,DemoTitleStyle]}]
,None}


(* ::Section::Closed:: *)
(*Functions*)


(* ::Subsection::Closed:: *)
(*Example result*)


(* ::Text:: *)
(*To exercise some of the functions below it is useful to have a set of test data, as would be returned by QuestPlus. We define the test data here. The structure is a list of quantized parameter estimates, followed by a list of trials. Each trial is a list whose first element is a list of stimulus parameters, and whose second element is the outcome.*)


(* ::Input::Initialization:: *)
ExampleResult={{-20,3.5`,0.5`,0.02`},{{{-18},2},{{-22},1},{{-12},2},{{-13},2},{{-15},2},{{-16},2},{{-17},2},{{-18},2},{{-19},2},{{-20},2},{{-22},2},{{-25},1},{{-19},1},{{-16},2},{{-17},2},{{-18},2},{{-18},2},{{-18},1},{{-16},2},{{-17},2},{{-17},2},{{-17},2},{{-18},2},{{-18},2},{{-18},2},{{-18},2},{{-18},2},{{-19},2},{{-19},2},{{-19},2},{{-19},2},{{-19},2}}};


(* ::Subsection::Closed:: *)
(*SimulatedObserver*)


(* ::Input::Initialization:: *)
SimulatedObserver::usage="SimulatedObserver[x_List,psi_,parameters_List]\n This function simulates an observer governed by a particular psychometric function psi, with a list of parameters. The first argument is a list (possibly of length 1) of stimulus parameters. The output is the response, an integer in the range 1 to n, where n is the number of possible responses, as determined by psi.";


(* ::Input::Initialization:: *)
SimulatedObserver[x_List,psi_,parameters_List]:=
Position[RandomVariate[MultinomialDistribution[1,psi[x,parameters]]],1][[1,1]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Text:: *)
(*Here is an example of one trial at intensity 2, for an observer governed by a Weibull psychometric function with a threshold at 0, a Weibull slope of 3, a guessing parameter of 0, and a lapsing parameter of 0.1.*)


(* ::Input:: *)
(*SimulatedObserver[{2},QpPFWeibull,{0,3,0,.01}]*)


(* ::Input:: *)
(*SimulatedObserver[{100,100,-.04,.02},QpPFLongest1,{.016,.47,.01,0,70,23,12}]*)


(* ::Text:: *)
(*Here is an example, simulating 100 trials at intensities from -9 to 9, for an observer with a threshold at 0 and a Weibull slope of 3.*)


(* ::Input:: *)
(*n=100;*)
(*ListPlot[Total[Table[{x,SimulatedObserver[{x},QpPFWeibull,{0,3,0,.01}]-1},{n},{x,-9,9}]]/n,QpStyles]*)


(* ::Input:: *)
(*n=100;*)
(*ListPlot[Total[Table[{t,SimulatedObserver[{100,100,-.04,t},QpPFLongest1,{.016,.47,.01,0,70,23,12}]-1},{n},{t,-.04,.04,.01}]]/n,QpStyles]*)


(* ::Subsection::Closed:: *)
(*UnitizeArray*)


(* ::Input:: *)
(*UnitizeArray::usage = "UnitizeArray[v_]\nScale an array to have a total of 1. If the total is initially zero, a uniform array with total 1 is returned.";*)


(* ::Input::Initialization:: *)
UnitizeArray[v_] := Block[{total},
total=Total[v,Infinity];
If[total>0
,v/total
,UniformArray[Dimensions[v]]]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*UnitizeArray[ConstantArray[0,{2,2}]]*)


(* ::Input:: *)
(*UnitizeArray[{{1,2},{3,4}}]*)


(* ::Subsection::Closed:: *)
(*UniformArray*)


UniformArray::usage="UniformArray[dimensions_List]\nCreate a constant array with a total of 1.";


(* ::Input::Initialization:: *)
UniformArray[dimensions_List] := ConstantArray[1./(Times @@ dimensions),dimensions]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*UniformArray[{3,4}]*)


(* ::Subsection::Closed:: *)
(*ArrayEntropy*)


(* ::Input::Initialization:: *)
ArrayEntropy::usage="ArrayEntropy[array_List, base_:2]\n Compute the zero-order entropy of an array of values, using the selected base logarithm.";


(* ::Input::Initialization:: *)
ArrayEntropy[array_List,base_:2] :=-Total[(# Log[base,#])&@DeleteCases[Flatten[array],0 | 0.]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*ArrayEntropy[RandomReal[{0,1},{10,10}]]*)


(* ::Subsection::Closed:: *)
(*ListArgMin*)


(* ::Code::Initialization:: *)
ListArgMin::usage = "ListArgMin[list_]\nReturn the location of the smallest element in a list or array.";


(* ::Code::Initialization:: *)
ListArgMin[list_] :=FirstPosition[list,Min[list]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*ListArgMin[{{2,3,4},{1,2,6}}]*)


(* ::Subsection::Closed:: *)
(*ListArgMax*)


(* ::Input::Initialization:: *)
ListArgMax::usage = "ListArgMax[list_]\nReturn the location of the largest element in a list or array.";


(* ::Input::Initialization:: *)
ListArgMax[list_] :=FirstPosition[list,Max[list]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*ListArgMax[{{2,3,4},{1,2,6}}]*)


(* ::Subsection::Closed:: *)
(*QuestPlus *)


(* ::Input::Initialization:: *)
QuestPlus::usage = "QuestPlus[ntrials_,options___Rule]\nRun an experiment of \!\(\*
StyleBox[\"ntrials\",\nFontSlant->\"Italic\"]\) trials using the QUEST+ method. The options determine the configuration of the experiment.";


(* ::Input::Initialization:: *)
Options[QuestPlus] = {QpOutcome->(SimulatedObserver[#,QpPFWeibull,{-20,3.5,.5,.02}]&)
,QpPNames->{"Threshold (dB)","Slope","Guess","Lapse"}
,QpXNames->{"Contrast (dB)"}
,QpXSamples->{Range[-40,0]}
,QpPSamples->{Range[-40,0],{3.5},{.5},{0.02}}
,QpPsi->QpPFWeibull
,QpPrior->"Constant"
,QpColors-> {Red,Green,Blue,Orange,Purple,Brown,Pink,Cyan,Gray,Black}
,Verbose->False};


(* ::Input::Initialization:: *)
QuestPlus[ntrials_,options___Rule] := Block[
{pnames,xsamples(*,psamples*),psi,outcome,prior,response,data,depth,nxd,npd,nld,likelihood,likexprior,probability,posteriors,entropies,nextindex,xnext,(*posterior,*)verbose},
{xsamples,psamples,psi,response,prior,verbose} = {QpXSamples,QpPSamples,QpPsi,QpOutcome,QpPrior,Verbose} /. {options} /. Options[QuestPlus];

data = {};
QpSeconds=0;
nxd=Length[xsamples];
npd=Length[psamples];
nld = nxd+npd+1;
depth = nxd+1;
If[prior ==="Constant",
prior= UniformArray[Length /@ psamples]];
prior=Developer`ToPackedArray[prior];
likelihood=Chop[N[Outer[psi[Take[{##},nxd],Drop[{##},nxd]]&,Sequence @@ Join[xsamples, psamples]]]];
likelihood=Transpose[likelihood,RotateLeft[Range[nld]]];
likelihood=Developer`ToPackedArray[likelihood];
If[verbose,Print["samples = ", Times @@ Dimensions[likelihood]]];
posterior = prior;
Do[(
likexprior=Map[(# posterior)& ,likelihood,{depth}];
likexprior=Developer`ToPackedArray[likexprior];
probability= Map[Total[#,Infinity]& , likexprior,{depth}];
posteriors= Map[UnitizeArray , likexprior,{depth}]  ;
entropies=Total[MapThread[
(ArrayEntropy[#1] #2)&,{posteriors,probability},depth]];
nextindex=ListArgMin[entropies];
xnext = MapThread[#1[[#2]]&,{xsamples,nextindex}];
outcome=response[xnext];
data = Append[data,{xnext,outcome}];
posterior= posteriors[[outcome,Sequence @@ nextindex]];
)
,{ntrials}];
If[verbose,Print[" seconds = ",QpSeconds]];
{MapThread[#1[[#2]]&,{psamples,ListArgMax[posterior]}],data}]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*result=QuestPlus[32];*)
(*QpPlot[result]*)


(* ::Input:: *)
(*result*)


(* ::Input:: *)
(*Dimensions[posterior]*)


(* ::Input:: *)
(*FindMinimum[(Total[If[#>10^x,#,0]&/@Flatten[posterior]]-.95)^2,{x,-1,-2}][[2,1,2]]*)


(* ::Input:: *)
(*posns=Position[posterior,#]&/@Select[Flatten[posterior],#>10^%&]*)


(* ::Input:: *)
(*Table[MapThread[#1[[#2]]&,{psamples,posns[[i,1]]}],{i,Length[posns]}]*)


(* ::Subsection::Closed:: *)
(*QpData*)


(* ::Input::Initialization:: *)
QpData::usage = "QpData[trials_]\nConverts a sequence of QuestPlus trials to a sorted list of {list of stimulus values, list of outcomes} pairs. A list of trials is the second element of the list returned by QuestPlus.";


(* ::Input::Initialization:: *)
QpData[trials_]:=Sort[({#1[[1,1]],#1[[All,2]]}&)/@GatherBy[trials,First]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*trials=ExampleResult[[2]]*)


(* ::Input:: *)
(*QpData[trials]*)


(* ::Subsection::Closed:: *)
(*QpCounts*)


(* ::Input::Initialization:: *)
QpCounts::usage="QpCounts[data_,noutcomes_]\nConverts the output of QpData to a list of {list of stimulus values, {count of outcome-1, count of outcome-2,...,count of outcome-n}} pairs. The argument \!\(\*
StyleBox[\"noutcomes\",\nFontSlant->\"Italic\"]\) is the number of possible outcomes.";


(* ::Input::Initialization:: *)
QpCounts[data_,noutcomes_] := {#[[1]],Function[x,Count[#[[2]],x]] /@ Range[noutcomes]}& /@ data


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*trials=ExampleResult[[2]]*)


(* ::Input:: *)
(*QpData[trials]*)


(* ::Input:: *)
(*QpCounts[QpData[trials],2]*)


(* ::Subsection::Closed:: *)
(*QpProportions*)


(* ::Input::Initialization:: *)
QpProportions::usage = "QpProportions[data_,noutcomes_]\nConvert the output of QpData to a list of pairs of the form {list of stimulus values, proportion of each outcome}.  The argument \!\(\*
StyleBox[\"noutcomes\",\nFontSlant->\"Italic\"]\) is the number of possible outcomes.";


(* ::Input::Initialization:: *)
QpProportions[data_,noutcomes_]:= {#[[1]],#[[2]]/Total[#[[2]]]}& /@ QpCounts[data,noutcomes]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*QpProportions[QpData[trials],2]*)


(* ::Subsection::Closed:: *)
(*QpLogLikelihood*)


(* ::Input::Initialization:: *)
QpLogLikelihood::usage = "QpLogLikelihood[data_,psi_,parameters_List]\nCompute the log likelihood of a set of data, given a psychometric function and a set of parameters. Data is a list of the form returned by QpCounts.";


(* ::Input:: *)
(*Clear[QpLogLikelihood]*)


(* ::Input::Initialization:: *)
QpLogLikelihood[counts_,psi_,parameters_List]:=Block[{p},
p=  N[psi[#,parameters]]& /@ counts[[All,1]];
Total[MapThread[QpNLogP,{counts[[All,2]],p},2],Infinity]]


(* ::Text:: *)
(*This is an older version.*)


(* ::Input:: *)
(*QpLogLikelihood[counts_,psi_,parameters_List]:=Block[{p,logp},*)
(*p=  N[psi[#,parameters]]& /@ counts[[All,1]];*)
(*logp=Log[p];*)
(*Total[logp counts[[All,2]],Infinity]]*)


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Text:: *)
(*Here we pull the counts out of our test data and compute the log likelihood for one set of parameters.*)


(* ::Input:: *)
(*counts = QpCounts[QpData[ExampleResult[[2]]],2];*)
(*QpLogLikelihood[counts,QpPFWeibull,{-20,3,0.5`,0.02`}]*)


(* ::Subsection::Closed:: *)
(*QpNLogP*)


(* ::Input::Initialization:: *)
QpNLogP[n_,p_] := If[p>0,n Log[p],If[n>0,-$MaxMachineNumber,0]]


(* ::Input:: *)
(*QpNLogP[.1,0.]*)


(* ::Subsection::Closed:: *)
(*PDFVonMises*)


(* ::Input::Initialization:: *)
PDFVonMises::usage = "PDFVonMises[x_,{mean_, concentration_}]\n Probability density function of a Von Mises distribution.";


(* ::Input::Initialization:: *)
PDFVonMises[x_,{mean_, concentration_}] :=  Exp[concentration Cos[x-mean]]/(2 Pi BesselI[0,concentration])


(* ::Subsubsection::Closed:: *)
(*Examples*)


(* ::Input:: *)
(*Plot[PDFVonMises[x,{1,3}],{x,0,2 \[Pi]}*)
(*,FrameTicks->{{Automatic,None},{{0,Pi,2Pi},None}}*)
(*,Evaluate@QpStyles]*)


(* ::Subsection::Closed:: *)
(*CDFVonMises*)


(* ::Input:: *)
(*Clear[CDFVonMises]*)


(* ::Input::Initialization:: *)
CDFVonMises::usage = "CDFVonMises[x_,{mean_, concentration_}]\n Cumulative probability density function of a Von Mises distribution that has support {0, 2 Pi}. Returns the Integral of the PDF from 0 to x.\nCDFVonMises[{x1_,x2_},{mean_, concentration_}]\n Returns the total density bewteen x1 and x2.";


(* ::Input::Initialization:: *)
CDFVonMises[x_,{mean_, concentration_}] :=  NIntegrate[PDFVonMises[y,{mean, concentration}],{y,0,x}]


(* ::Input::Initialization:: *)
CDFVonMises[{x1_,x2_},{mean_, concentration_}] :=  CDFVonMises[x2,{mean, concentration}]-CDFVonMises[x1,{mean, concentration}]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*Manipulate[*)
(*Plot[{PDFVonMises[x,{mean, concentration}],CDFVonMises[x,{mean, concentration}]},{x,0,2 Pi}*)
(*,PlotRange->{Full,Full}*)
(*,FrameTicks->{{None,None},{{0,Pi, 2 Pi},None}}*)
(*,Filling->Axis*)
(*,Evaluate@QpStyles]*)
(*,{{mean,Pi},0,2 Pi}*)
(*,{{concentration,1},0,10}*)
(*,ContinuousAction->False*)
(*,Evaluate[Sequence @@DemoStyle]]*)


(* ::Subsubsection::Closed:: *)
(*Alternates*)


(* ::Input:: *)
(*CDFVonMises[x_,{mean_, concentration_},dx_:0.01] :=dx  Total[Table[PDFVonMises[y,{mean, concentration}],{y,0,x,dx}]]*)


(* ::Input:: *)
(*CDFVonMises[{x1_,x2_},{mean_, concentration_}] :=  NIntegrate[PDFVonMises[y,{mean, concentration}],{y,x1,If[x2>x1,x2,x2+2 Pi]}]*)


(* ::Subsection::Closed:: *)
(*QpFit*)


(* ::Input::Initialization:: *)
QpFit::usage = "QpFit[result_,options___Rule]\n Fits a psychometric function to a set of data. The input is the structure returned by QuestPlus. It responds to the QuestPlus options QpPSamples and QpPsi. The function accepts any number of outcomes and of stimulus dimensions.";


(* ::Input:: *)
(*Clear[QpFit]*)


(* ::Input::Initialization:: *)
QpFit[result_,options___Rule]:= Block[{fit,names,psamples,constraints,psi,pbounds,counts,noutcomes},
{psamples,psi} = {QpPSamples,QpPsi} /. {options} /. Options[QuestPlus];
noutcomes=Max[result[[2,All,2]]];
pbounds = MinMax /@ psamples;
counts = QpCounts[QpData[result[[2]]],noutcomes];
names = Symbol["QpTmp"<>ToString[#]]& /@ Range[Length[pbounds]];
constraints=MapThread[(#2[[1]]<= #1<= #2[[2]])&,{names,pbounds}];
fit=NMaximize[{QpLogLikelihoodNumeric[counts,psi,names],constraints},names,WorkingPrecision->10];
(*Print[fit];*)
names /.fit[[2]]]


(* ::Text:: *)
(*This function is only required for the case of QpPFCircular, due to limitations of NMaximize. It may slow things down for other psychometric functions.*)


(* ::Input::Initialization:: *)
QpLogLikelihoodNumeric[counts_,psi_,names_?(VectorQ[#,NumericQ]&)] := QpLogLikelihood[counts,psi,names]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*QpFit[ExampleResult]*)


(* ::Input:: *)
(*QpFit[ExampleResult,QpPSamples->{Range[-40,0,.01],Range[8],{.5},Range[.00,.04,.01]}]*)


(* ::Subsection::Closed:: *)
(*PiePoint*)


(* ::Input::Initialization:: *)
PiePoint::usage = "PiePoint[record_,sizes_,max_,min_]\nDraws a data point as a pie chart.
sizes: a list of horizontal and vertical radii
max: the largest count
min: the smallest proportion.";


(* ::Input:: *)
(*Clear[PiePoint]*)


(* ::Input::Initialization:: *)
PiePoint[record_,sizes_List,max_,min_] := Block[{tsize,pc},
tsize = sizes Max[min,Total[record[[2]]]/max]^.5;
pc =record[[2,2]]/Total[record[[2]]]; MapThread[{#1,Disk[record[[1]],tsize,#2]}&,{{Red,Green},2Pi{{0,pc-1},{0,pc}}}]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*tst={{{0,-40},{1,0}},{{0,-38},{0,1}},{{0,-36},{1,1}},{{0,-34},{1,1}},{{0,-32},{0,2}},{{18,-26},{0,6}},{{20,-26},{1,2}},{{22,-26},{0,1}},{{34,0},{0,1}},{{36,0},{0,1}},{{38,0},{0,3}},{{40,-6},{1,0}},{{40,-4},{1,0}},{{40,-2},{0,1}},{{40,0},{1,5}}};*)


(* ::Input:: *)
(*Graphics[PiePoint[#,{1,1},9,.1]& /@ tst,AspectRatio->1]*)


(* ::Subsection::Closed:: *)
(*PieMarkers*)


(* ::Input::Initialization:: *)
PieMarkers[n_,colors_] := Table[Graphics[{colors[[k]],Disk[{0,0},1,(2Pi/n){k-1,k}],Opacity[0],Disk[{0,0},1]}],{k,n}]


(* ::Subsection::Closed:: *)
(*QpPlot*)


(* ::Input::Initialization:: *)
QpPlot::usage = "QpPlot[result_,options___Rule]\nPlots the results of a QuestPlus experiment for the case of one stimulus dimension. The input is the output of QuestPlus. It takes the same options as QuestPlus. The function first fits a psychometric function and then plots the result. The quantized parameter estimates from QuestPlus, and from the un-quantized fit are shown. The blue points are the data, and saturation indicates the relative number of trials.";


(* ::Input::Initialization:: *)
QpPlot[result_,options___Rule] := Block[
{trialnumbers,trialdensity,pc,xsamples,psi,pnames,xnames,xbounds,estimates,data,psamples,pbounds,fit},
{xsamples,psi,pnames,xnames} = {QpXSamples,QpPsi,QpPNames, QpXNames} /. {options} /. Options[QuestPlus];
xbounds=MinMax[xsamples];
estimates=result[[1]];
data = QpData[result[[2]]];
trialnumbers = Length /@ data[[All,2]];
trialdensity = trialnumbers/Max[trialnumbers];
pc =(Flatten /@QpProportions[data,2])[[All,{1,3}]];
fit =Chop[QpFit[result,options]];
Plot[psi[{x},fit][[2]] ,{x,xbounds[[1]],xbounds[[2]]}
,Epilog->{Blue,AbsolutePointSize[8],
MapThread[{Opacity[Max[.1,#2]],Point [#1]}&,{ pc,trialdensity}]
,Gray
,Inset[NumberForm[Grid[Join[
{{"","Q+","Fit"}},
Transpose[{pnames, estimates, fit}]]],3],Scaled[{.95,.05}],{Right,Bottom}]}
,PlotRangeClipping->False
,PlotRange->{xbounds,{-0,1.}}
,PlotStyle->{AbsoluteThickness[2],Red}
,FrameLabel->{xnames[[1]],"Probability"}
,Evaluate[Sequence @@QpStyles]]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*QpPlot[ExampleResult]*)


(* ::Subsection::Closed:: *)
(*QpPlot2D*)


(* ::Input::Initialization:: *)
QpPlot2D::usage = "QpPlot2D[result_,options___Rule]\nPlots the results of a QuestPlus experiment for the case of two stimulus dimensions. The input is the output of QuestPlus. It takes the same options as QuestPlus. The function first fits a psychometric function and then plots the result. The quantized parameter estimates from QuestPlus, and from the un-quantized fit are shown. Each data point is a pie chart with a size reflecting the number of trials and green and red sectors indicating proportions of outcomes. There are two options: QpPointSize\[Rule]1, indicating the relative size of the data points, and QpPMin\[Rule]0.05, the minimum relative size.";


(* ::Input::Initialization:: *)
Options[QpPlot2D] = {QpPointSize->1,QpPMin->0.05};


(* ::Input:: *)
(*Clear[QpPlot2D]*)


(* ::Input::Initialization:: *)
QpPlot2D[result_,options___Rule] := Block[{max,ranges,counts,xsamples,xnames,pnames,outcome,size,min,psi,param,image,psifig,estimates,fit,sizes},
{xsamples,xnames,outcome,size,min,pnames} = {QpXSamples,QpXNames,QpOutcome,QpPointSize,QpPMin,QpPNames} /. {options} /. Options[QuestPlus]/. Options[QpPlot2D];
counts = QpCounts[QpData[result[[2]]],2];
max= Max[counts[[All,2]]];
ranges = MinMax /@ xsamples;
sizes =- size ((Subtract@@#)& /@ ranges)/20.;

psi = outcome[[1,2]];
estimates=result[[1]];
(*estimates=outcome[[1,3]];*)
fit =QpFit[result,options];
image= Table[psi[{x,y},fit][[2]],{y,xsamples[[2]]},{x,xsamples[[1]]}];
psifig=ArrayPlot[Reverse[image]
,ColorFunction->GrayLevel
,ColorFunctionScaling->False
,DataRange->ranges];
ListPlot[{{0,0}}
,PlotStyle->Opacity[0]
,Prolog->psifig[[1]]
,Epilog->{PiePoint[#,sizes,max,min]&/@counts,Red,
Inset[NumberForm[Grid[Join[
{{"","Q+","Fit"}},
Transpose[{pnames, estimates, fit}]]],3],Scaled[{.95,.05}],{Right,Bottom}]}
,Joined->False
,FrameLabel->xnames
,PlotRange->ranges
,PlotRangeClipping->False
,PlotRangePadding->Scaled[.07]
,AspectRatio->1
,QpStyles]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*options=Sequence[*)
(*QpOutcome-> (SimulatedObserver[#,QpPFSCSF,{-35,-50,1.2}]& )*)
(*,QpPNames->{"t","\!\(\*SubscriptBox[\(c\), \(0\)]\)","\!\(\*SubscriptBox[\(c\), \(f\)]\)"}*)
(*,QpXNames->{"Spatial frequency (cycles/deg)","Contrast (dB)"}*)
(*,QpXSamples->{Range[0,40,2],Range[-50,0,2]}*)
(*,QpPSamples->{Range[-30,-50,-2],Range[-40,-60,-2],Range[.8,1.6,.2]}*)
(*,QpPsi->QpPFSCSF];*)


(* ::Input:: *)
(*result = {{-32,-56,1.4},{{{40,0},2},{{40,-4},1},{{34,0},2},{{36,0},2},{{0,-38},2},{{38,0},2},{{0,-40},1},{{38,0},2},{{18,-26},2},{{18,-26},2},{{40,0},2},{{0,-36},2},{{0,-36},1},{{40,0},2},{{20,-26},2},{{20,-26},2},{{40,-2},2},{{22,-26},2},{{40,-6},1},{{20,-26},1},{{40,0},2},{{18,-26},2},{{40,0},2},{{18,-26},2},{{0,-32},2},{{0,-32},2},{{0,-34},2},{{0,-34},1},{{40,0},1},{{18,-26},2},{{38,0},2},{{18,-26},2}}};*)


(* ::Input:: *)
(*QpPlot2D[result,options]*)


(* ::Subsection::Closed:: *)
(*QpPlot3D*)


(* ::Input::Initialization:: *)
QpPlot3D::usage = "QpPlot3D[result_,options___Rule]\nPlots the results of a QuestPlus experiment for the case of three stimulus dimensions. The input is the output of QuestPlus. It takes the same options as QuestPlus. The function first fits a psychometric function and then plots the result. The quantized parameter estimates from QuestPlus, and from the un-quantized fit are shown. Each data point is a pie chart with a size reflecting the number of trials and green and red sectors indicating proportions of outcomes. There are two options: QpPointSize\[Rule]3, indicating the relative size of the data points, and QpPMin\[Rule]0.05, the minimum relative size.";


(* ::Input::Initialization:: *)
Options[QpPlot3D] = {QpPointSize->3,QpPMin->0.05};


(* ::Input::Initialization:: *)
QpPlot3D[result_,options___Rule] := Block[{max,counts,size,min,xsamples,xnames,outcome,ranges,images,psifigs,pics,fit,pnames,estimates,start,sizes},
{xsamples,xnames,outcome,pnames,size,min} = {QpXSamples,QpXNames,QpOutcome,QpPNames,QpPointSize,QpPMin} /. {options} /. Options[QuestPlus] /. Options[QpPlot3D];
ranges = MinMax /@ xsamples;
sizes =- size ((Subtract@@#)& /@ Rest[ranges])/40.;

counts = QpCounts[QpData[result[[2]]],2];
max= Max[counts[[All,2]]];
fit =QpFit[result,options];
estimates=result[[1]];


images= Table[QpPFSTCSF[{c,f, w},outcome[[1,3]]][[2]],{c,xsamples[[1]]},{w,xsamples[[3]]},{f,xsamples[[2]]}];
start=True;
psifigs=MapThread[ListDensityPlot[#1
,InterpolationOrder->0
,ColorFunction->GrayLevel
,ColorFunctionScaling->False
,DataRange->Rest[ranges]
,FrameLabel->None
,Epilog->{White,Style[Text[#2,Scaled[{.93,.93}],{1,1}],24]
,If[start,(start=False;Inset[NumberForm[Grid[Join[
{{"","Q+","Fit"}},
Transpose[{pnames, estimates, fit}]]],3],Scaled[{.95,.05}],{Right,Bottom}]),{}]}
,ImageSize->{Automatic,180}
,QpStyles]& ,{ images,xsamples[[1]]}];
pics=MapThread[Function[{x,y},Show[y,Graphics[PiePoint[{Rest[#[[1]]],#[[2]]},sizes,max,.2]& /@ Cases[counts,{{x,__},__}]]
,PlotRange->((#+{-2,2})& /@ Rest[ranges])] ],{ xsamples[[1]],psifigs}];
Labeled[GraphicsGrid[Partition[Show[#,FrameLabel->None]& /@ pics,3]],MapAt[Rotate[#,Pi/2]&,Style[#,24,FontFamily->"Helvetica"]& /@ Rest[xnames],2],{Bottom,Left}]
]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*options=Sequence[*)
(*QpOutcome-> (SimulatedObserver[#,QpPFSTCSF,{-35,-50,1.2,1.0}]& )*)
(*,QpPNames->{"t","\!\(\*SubscriptBox[\(c\), \(0\)]\)","\!\(\*SubscriptBox[\(c\), \(f\)]\)","\!\(\*SubscriptBox[\(c\), \(w\)]\)"}*)
(*,QpXNames->{"Contrast (dB)","Spatial frequency (c/d)","Temporal frequency (Hz)"}*)
(*,QpXSamples->{Range[-40,0,5],Range[0,40,5],Range[0,40,5]}*)
(*,QpPSamples->{Range[-30,-50,-5],Range[-40,-60,-5],Range[.8,1.6,.2],Range[.8,1.6,.2]}*)
(*,QpPsi->QpPFSTCSF];*)


(* ::Input:: *)
(*result = {{-35,-50,1.2000000000000002`,1.`},{{{0,0,40},2},{{0,40,0},2},{{0,5,40},2},{{0,40,5},2},{{-10,0,40},2},{{-10,40,0},2},{{0,10,40},2},{{-15,40,0},2},{{-15,40,0},1},{{0,40,5},1},{{-10,0,40},1},{{0,0,40},2},{{0,35,0},2},{{0,0,40},2},{{0,40,0},2},{{0,40,0},2},{{-40,0,0},1},{{0,0,40},2},{{0,40,0},2},{{0,5,40},2},{{0,5,40},2},{{-25,20,0},1},{{-35,0,0},2},{{-35,0,0},1},{{0,40,0},2},{{0,40,0},2},{{0,5,40},2},{{0,40,0},2},{{0,5,40},2},{{-25,5,15},2},{{-25,5,15},2},{{-25,5,15},2},{{-25,5,15},2},{{-25,5,15},2},{{-25,0,20},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{-15,0,40},1},{{0,10,40},2},{{0,10,40},1},{{0,40,5},2},{{0,40,5},2},{{0,40,5},2},{{0,40,5},2},{{0,40,5},1},{{0,10,40},2},{{0,10,40},1},{{-25,5,15},2},{{-25,5,15},2},{{-25,5,15},2},{{-30,0,0},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{-30,15,5},2},{{-30,15,5},2},{{-30,15,5},2},{{-30,15,5},1},{{-30,0,15},2},{{0,10,40},1},{{-30,0,15},2},{{0,5,40},2},{{-15,30,0},1},{{0,35,5},2},{{-30,0,15},2},{{0,35,5},2},{{-30,0,15},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},1},{{0,35,5},2},{{-30,0,15},2},{{-30,0,15},2},{{0,35,5},2},{{-30,0,15},2},{{-30,0,15},2},{{0,40,0},1},{{0,35,5},2},{{0,35,5},2},{{0,35,5},2},{{0,35,5},2},{{0,35,5},2},{{0,40,0},2},{{0,5,40},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{-30,0,15},2},{{0,40,0},2},{{0,5,40},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{-30,0,15},2},{{0,5,40},2},{{0,40,0},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},1},{{-30,0,15},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{0,40,0},2},{{-30,10,10},2},{{-30,10,10},1},{{-30,0,15},2},{{-30,0,15},2},{{0,5,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},1},{{-30,0,15},2},{{-30,0,15},2},{{-5,5,40},2},{{-5,5,40},2},{{-5,5,40},2},{{-5,5,40},1},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},2},{{0,10,40},1},{{-35,0,0},1},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,0,15},2},{{-30,10,10},2},{{-30,10,10},2},{{-30,10,10},1},{{-30,0,15},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{0,40,5},2},{{0,40,5},2},{{0,40,5},1},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{-5,5,40},2},{{-5,5,40},2},{{-5,5,40},2},{{0,10,40},1},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{-30,10,10},2},{{-30,10,10},1},{{-30,0,15},2},{{-30,0,15},2},{{0,40,0},1},{{0,40,0},2},{{0,40,0},2},{{0,40,0},2},{{0,40,0},2},{{0,40,0},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{0,5,40},2},{{0,40,0},2},{{-30,0,15},2},{{0,40,0},2},{{-30,0,15},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},1},{{0,5,40},2},{{0,40,0},2},{{0,40,0},2},{{-5,5,40},2},{{-5,5,40},1},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{-30,0,15},2},{{0,5,40},2},{{-30,0,15},2},{{0,5,40},1},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{0,5,40},2},{{-30,0,15},2},{{0,5,40},2},{{-30,0,15},2},{{0,5,40},2},{{-35,0,0},2},{{-35,0,5},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},2},{{-35,0,0},1},{{0,5,40},2},{{-10,35,0},2},{{-10,35,0},2},{{-10,35,0},2},{{-10,35,0},1},{{0,5,40},2},{{0,40,0},2},{{-30,0,15},2},{{0,40,0},2},{{-30,15,5},1},{{-30,0,15},2},{{-30,0,15},2},{{-5,5,40},2}}};*)


(* ::Input:: *)
(*QpPlot3D[result,options]*)


(* ::Subsection::Closed:: *)
(*QpPlotCategories*)


(* ::Input::Initialization:: *)
QpPlotCategories::usage = "QpPlotCategories[result_,options___Rule]\nPlots the results of a QuestPlus experiment for the case of categorization. The input is the output of QuestPlus. It takes the same options as QuestPlus. The function first fits a psychometric function and then plots the result. The quantized parameter estimates from QuestPlus, and from the un-quantized fit are shown. Each data points are sectors of a pie, color coded to indicate the category. The opacity indicates the numebr of trials.";


(* ::Input:: *)
(*Clear[QpPlotCategories]*)


(* ::Input::Initialization:: *)
QpPlotCategories[result_,options___Rule] :=Block[
{xnames,xsamples,psamples,psi,noutcomes,colors,pnames,fit,top,xvals,pvals,mfig,markers,pc,densities,dfig,estimates} ,
{xnames,pnames,xsamples,psi,colors} = {QpXNames,QpPNames,QpXSamples,QpPsi,QpColors} /. {options} /. Options[QuestPlus];
noutcomes=Max[result[[2,All,2]]];
fit=QpFit[result,options];
estimates=result[[1]];
top=xsamples[[1,-1]];
xvals=top Range[0,1,.01];
pvals= psi[{#},fit]& /@ xvals;
mfig=ListLinePlot[ Transpose[pvals]
,PlotStyle->colors
,PlotRange->{-.05,1.05}
,DataRange->{0,top}
,FrameLabel->{xnames[[1]],"Probability"}
,PlotLegends->LineLegend[Range[noutcomes],LegendLabel->"Outcomes"]
,Epilog->{Red,
Inset[NumberForm[Grid[Join[
{{"","Q+","Fit"}},
Transpose[{pnames, estimates, fit}]]],3],Scaled[{1,.05}],{Right,Bottom}]}
,Evaluate@QpStyles];
markers = PieMarkers[noutcomes,colors];
pc=QpProportions[QpData[result[[2]]],noutcomes];
densities = Length /@ QpData[result[[2]]][[All,2]];
densities = densities/Max[densities];
dfig=Show[Table[ListPlot[List /@ ({#[[1,1]],#[[2,k]]}& /@ pc)
,PlotMarkers->{markers[[k]],.07}
,PlotRange->{-.05,1.05}
,PlotStyle->(Directive[AbsolutePointSize[8],colors[[k]],Opacity [#]]&/@ densities)
,PlotRangeClipping->False
,QpStyles],{k,noutcomes}]];
Show[mfig,dfig]
]


(* ::Input:: *)
(*QpFit[result,options]*)


(* ::Input:: *)
(*QpPlotCategories[result,options]*)


(* ::Subsection::Closed:: *)
(*QpInitialize *)


(* ::Input::Initialization:: *)
QpInitialize::usage = "QpInitialize[options___Rule]\nInitialize the data structure for a QUEST+ experiment. The options determine the configuration of the experiment. The function returns a large data structure consisting of the elements: {data, posterior, posteriors, nextindex, xnext, depth, likelihood, {options}}. When there is more than one condition, a structure can be created for each.";


(* ::Input::Initialization:: *)
QpInitialize[options___Rule] := Block[
{pnames,xsamples,psamples,psi,outcome,prior,response,data,depth,nxd,npd,nld,likelihood,likexprior,probability,posteriors,entropies,nextindex,xnext,posterior,verbose},

{xsamples,psamples,psi,response,prior,verbose} = {QpXSamples,QpPSamples,QpPsi,QpOutcome,QpPrior,Verbose} /. {options} /. Options[QuestPlus];

data = {};
nxd=Length[xsamples];
npd=Length[psamples];
nld = nxd+npd+1;
depth = nxd+1;
If[prior ==="Constant",
prior= UniformArray[Length /@ psamples]];
prior=Developer`ToPackedArray[prior];
likelihood=Chop[N[Outer[psi[Take[{##},nxd],Drop[{##},nxd]]&,Sequence @@ Join[xsamples, psamples]]]];
likelihood=Transpose[likelihood,RotateLeft[Range[nld]]];
likelihood=Developer`ToPackedArray[likelihood];
If[verbose,Print["samples = ", Times @@ Dimensions[likelihood]]];
posterior = prior;
xnext =nextindex=posteriors=0;
{data,posterior,posteriors,nextindex,xnext,depth,likelihood,{options}}
]


(* ::Subsection::Closed:: *)
(*QpQuery*)


(* ::Input::Initialization:: *)
QpQuery::usage = "QpQuery[structure_]\nQuery the QUEST+ data structure. The function updates the elements posteriors, nextindex, and xnext. The last is the vector of stimulus parameters that may be used to conduct the next trial. It returns the updated structure.";


(* ::Input::Initialization:: *)
QpQuery[structure_] := Block[
{data,depth,likelihood,likexprior,probability,posteriors,entropies,nextindex,xnext,posterior,options,xsamples,optionslist},
{data,posterior,posteriors,nextindex,xnext,depth,likelihood,optionslist}=structure;
{xsamples} = {QpXSamples} /. optionslist /. Options[QuestPlus];
likexprior=Map[(# posterior)& ,likelihood,{depth}];
likexprior=Developer`ToPackedArray[likexprior];
probability= Map[Total[#,Infinity]& , likexprior,{depth}];
posteriors= Map[UnitizeArray , likexprior,{depth}]  ;
entropies=Total[MapThread[
(ArrayEntropy[#1] #2)&,{posteriors,probability},depth]];
nextindex=ListArgMin[entropies];
xnext = MapThread[#1[[#2]]&,{xsamples,nextindex}];
{data,posterior,posteriors,nextindex,xnext,depth,likelihood,optionslist}]


(* ::Subsection::Closed:: *)
(*QpUpdate*)


(* ::Input::Initialization:: *)
QpUpdate::usage = "QpUpdate[structure_,outcome_]\nUsing the outcome from the previous trial, update the QUEST+ data structure. The function updates the elements data and posterior.";


(* ::Input::Initialization:: *)
QpUpdate[structure_,outcome_] := 
ReplacePart[structure,{1->Append[structure[[1]],{structure[[5]],outcome}],2->structure[[3]][[outcome,Sequence @@ structure[[4]]]]}]


(* ::Subsection::Closed:: *)
(*QpRun*)


(* ::Text:: *)
(*QpRun::usage = "QpRun[ntrials_,options___Rule]\nRun some trials of a QUEST+ experiment. This version illustrates the use of functions QpInitialize, QpQuery, and QpUpdate. It is an alternative to the integrated function QuestPlus. It takes the same options as QuestPlus.";*)


(* ::Input::Initialization:: *)
QpRun[ntrials_,options___Rule] :=Block[{response,outcome,structure,psamples},
{response,psamples} = {QpOutcome,QpPSamples} /. {options} /. Options[QuestPlus];
structure = QpInitialize[options];
Do[(
structure=QpQuery[structure];
outcome=response[structure[[5]]];
structure=QpUpdate[structure,outcome];)
,{ntrials}];
{MapThread[#1[[#2]]&,{psamples,ListArgMax[structure[[2]]]}],structure[[1]]}]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*result=QpRun[32]*)
(*QpPlot[result]*)


(* ::Subsection::Closed:: *)
(*QpCredibleParams*)


(* ::Input::Initialization:: *)
QpCredibleParams::usage = "QpCredibleParams[result_,percentile_,options___Rule]\n Returns a list of all parameter combinations within the percentile (e.g. 0.95) credible interval. The input is the structure returned by QuestPlus. It responds to the all the QuestPlus options. The function accepts any number of outcomes and of stimulus dimensions.";


(* ::Input::Initialization:: *)
QpCredibleParams[result_,percentile_,options___Rule]:= Block[{pnames,xsamples,psamples,psi,prior,response,depth,nxd,npd,nld,likelihood,likexprior,probability,posteriors,posterior,verbose,cutoff,posns},
{xsamples,psamples,psi,response,prior,verbose} = {QpXSamples,QpPSamples,QpPsi,QpOutcome,QpPrior,Verbose} /. {options} /. Options[QuestPlus];QpSeconds=0;
nxd=Length[xsamples];
npd=Length[psamples];
nld = nxd+npd+1;
depth = nxd+1;
If[prior ==="Constant",
prior= UniformArray[Length /@ psamples]];
prior=Developer`ToPackedArray[prior];
likelihood=Chop[N[Outer[psi[Take[{##},nxd],Drop[{##},nxd]]&,Sequence @@ Join[xsamples, psamples]]]];
likelihood=Transpose[likelihood,RotateLeft[Range[nld]]];
likelihood=Developer`ToPackedArray[likelihood];
If[verbose,Print["samples = ", Times @@ Dimensions[likelihood]]];
posterior = prior;
Do[(
likexprior=Map[(# posterior)& ,likelihood,{depth}];
likexprior=Developer`ToPackedArray[likexprior];
probability= Map[Total[#,Infinity]& , likexprior,{depth}];
posteriors= Map[UnitizeArray , likexprior,{depth}]  ;
posterior= posteriors[[result[[2,i,2]],Sequence @@ Table[Sequence@@FirstPosition[xsamples[[j]],result[[2,i,1,j]]],{j,Length[result[[2,i,1]]]}]]];
)
,{i,Length[result[[2]]]}];
If[verbose,Print[" seconds = ",QpSeconds]];
cutoff=FindMinimum[(Total[If[#>10^x,#,0]&/@Flatten[posterior]]-percentile)^2,{x,-1,-2}][[2,1,2]];
posns=Position[posterior,#]&/@Select[Flatten[posterior],#>10^cutoff&];
Table[MapThread[#1[[#2]]&,{psamples,posns[[i,1]]}],{i,Length[posns]}]]


(* ::Subsubsection::Closed:: *)
(*Example*)


(* ::Input:: *)
(*QpCredibleParams[{{-22,3.5`,0.5`,0.02`},{{{-18},2},{{-22},2},{{-25},1},{{-15},2},{{-16},2},{{-18},2},{{-20},2},{{-21},2},{{-22},2},{{-23},1},{{-19},2},{{-20},2},{{-20},2},{{-21},1},{{-19},2},{{-19},2},{{-20},2},{{-20},2},{{-20},2},{{-20},2},{{-21},2},{{-21},2},{{-21},2},{{-21},2},{{-21},2},{{-22},1},{{-21},2},{{-21},2},{{-21},2},{{-21},2},{{-21},1},{{-20},2}}},.95]*)


(* ::Section::Closed:: *)
(*Psychometric functions*)


(* ::Text:: *)
(*These are generic functions that are often used to fit psychometric data. Other more specialized psychometric functions are developed in the Examples section below.*)


(* ::Subsection::Closed:: *)
(*QpPFWeibull*)


(* ::Input::Initialization:: *)
QpPFWeibull[{x_},{threshold_,slope_,guess_,lapse_}] := 
{#,1-#}&@(lapse- (guess+lapse-1) Exp[- 10^(slope (x - threshold)/20) ])


(* ::Text:: *)
(*A plot of this function for one set of parameters shows the probability of the two outcomes (failure and success).*)


(* ::Input:: *)
(*Plot[Evaluate[QpPFWeibull[{x},{0,3,.5,.01}]],{x,-9,9},Evaluate@QpStyles]*)


(* ::Subsection::Closed:: *)
(*QpPFNormal*)


(* ::Input::Initialization:: *)
QpPFNormal[{x_},{mean_,sd_,lapse_}] :={1-#,#}&@(lapse + (1-2 lapse)CDF[NormalDistribution[mean,sd],x])


(* ::Input:: *)
(*Plot[Evaluate[QpPFNormal[{x},{0,3,.01}]],{x,-9,9},Evaluate@QpStyles]*)


(* ::Subsection::Closed:: *)
(*QpPFRating*)


(* ::Input::Initialization:: *)
QpPFRating::usage = "QpPFRating[{x_},s_]\nPschometric function for a rating experiment. There is one stimulus dimension, and a number of parameters equal to the number of outcomes. The argument s is a list of parameters consisting of the standard deviation followed by the \!\(\*
StyleBox[\"differences\",\nFontSlant->\"Italic\"]\) between several criteria.";


(* ::Input::Initialization:: *)
QpPFRating[{x_},s_] := Block[{vals,crit},
crit = Accumulate[Rest[s]];
vals=Chop[CDF[NormalDistribution[#,First[s]],x]& /@ crit];
vals=Append[-Differences[vals],Last[vals]];
Prepend[vals,1.-Total[vals]]
]


(* ::Subsubsection:: *)
(*Example*)


(* ::Text:: *)
(*This is a demonstration showing the effect of the number of categories and the standard deviation.*)


(* ::Input:: *)
(*Manipulate[( *)
(*xvals = Range[0,n+1,.01];*)
(*pvals= QpPFRating[{#},Flatten[{sd,1+0Range[n]}]]& /@ xvals;*)
(*ListLinePlot[ Transpose[pvals]*)
(*,PlotRange->{0,1}*)
(*,DataRange->{0,n+1}*)
(*,PlotLegends->Range[n+1],Evaluate@QpStyles])*)
(*,{n,1,10,1,ControlType->SetterBar}*)
(*,{sd,.1,2,Appearance->"Labeled"}]*)


(* ::Input:: *)
(*colors={Red,Green,Blue,Orange};*)
(*Plot[Evaluate@QpPFRating[{x},{.5,1,2,3}],{x,0,8}*)
(*,PlotStyle->colors*)
(*,PlotLegends->LineLegend[Range[4],LegendLabel->"Outcome"]*)
(*,FrameLabel->{"Magnitude","Probability"}*)
(*,Evaluate@QpStyles]*)


(* ::Text:: *)
(*Here we run some trials with a simulated observer and plot the results.*)


(* ::Input:: *)
(*ntrials=100;*)
(*n=3;*)
(*xvals = Range[0,n,.25];*)
(*res=Function[x,(run=Table[SimulatedObserver[{x},QpPFRating,{.25,1,1}],{ntrials}];Count[run,#]& /@ Range[n])] /@ xvals;*)
(*ListLinePlot[Transpose[res],DataRange->{0,n},PlotLegends->Range[n],QpStyles]*)


(* ::Subsection::Closed:: *)
(*QpPFCircular*)


(* ::Text:: *)
(*QpPFCircular::usage = "QpPFCircular[{x_},s_]\nPsychometric function for circular categorization, based on a von Mises distribution. The argument x is the stimulus parameter, which must be between 0 and 2 Pi. The argument s is a list of parameters of the form {concentration, first, width1, width2, ...widthn}, where first is the first criterion and the widths are the widths of the successive categories. The number of categories is the number of widths plus 1 (alternatively, the length of s minus 1).";*)


(* ::Input::Initialization:: *)
QpPFCircular[{x_},s_] :=  Block[{concentration,crit,first,widths,tmp},
concentration = First[s];
first = s[[2]];
widths = Drop[s,2];
crit = first + Accumulate[Prepend[widths,0]];
crit = Transpose[{crit,RotateLeft[crit]}];
tmp=CDFVonMises[#,{x, concentration}]& /@ Most[crit];
Append[tmp,1-Total[tmp]]
]


(* ::Text:: *)
(*Here we run some trials with a simulated observer and plot the results.*)


(* ::Input:: *)
(*ntrials=100;*)
(*n=3;*)
(*width = 2 Pi/n;*)
(*xvals = Range[0,15] Pi/8;*)
(*res=Function[x,(run=Table[SimulatedObserver[{x},QpPFCircular,{2,Pi/4,width,width}],{ntrials}];Count[run,#]& /@ Range[n])] /@ xvals;*)
(*ListLinePlot[Transpose[res],DataRange->{0,2 Pi},PlotLegends->Range[n]*)
(*,FrameTicks->{{None,None},{{0,Pi,2 Pi},None}},QpStyles]*)


(* ::Text:: *)
(*This demonstration shows the effect of varying the number of categories and the concentration.*)


(* ::Input:: *)
(*samples=20;*)
(*first=Pi/3;*)
(*Manipulate[( *)
(*colors = Hue /@ (Range[n]/n);*)
(*xvals = Range[0,2Pi,(2Pi/samples)];*)
(*widths=Table[2Pi/n,{n-1}];*)
(*s = {concentration,first,Sequence@@widths};*)
(*crit = first + Accumulate[Prepend[widths,0]];*)
(*crit = If[#>2Pi,#-2 Pi,#]& /@ crit;*)
(*pvals= QpPFCircular[{#},s]& /@ xvals;*)
(*ListLinePlot[ Transpose[pvals]*)
(*,PlotStyle->colors*)
(*,PlotRange->{0,1.02}*)
(*,DataRange->{0,2Pi}*)
(*,PlotLegends->Range[n]*)
(*,FrameTicks->{{Automatic,None},{{0,Pi, 2 Pi},None}}*)
(*,GridLines->{Range[0,2Pi,Pi/4],Automatic}*)
(*,Epilog->(MapThread[{Dashed,#1,Line[{{#2,0},{#2,1}}]}&, {colors, crit}])*)
(*,Evaluate@QpStyles])*)
(*,{n,2,10,1,ControlType->SetterBar}*)
(*,{{concentration,2,"Concentration"},.5,4,Appearance->"Labeled"}*)
(*,ContinuousAction->False]*)


(* ::Subsection::Closed:: *)
(*QpPFScaling*)


(* ::Text:: *)
(*We first define a sensory scale function. This yields zero for values of the stimulus below a threshold, and then rises as a power function of the stimulus less the threshold.*)


(* ::Input::Initialization:: *)
sensoryscale[x_,{max_,threshold_,power_}] := max (Max[0,x-threshold]/(1-threshold))^power


(* ::Text:: *)
(*We plot an example for particular values.*)


(* ::Input:: *)
(*s={max,threshold,power}={6, .2, 0.4};*)
(*Plot[sensoryscale[x,s],{x,0,1}*)
(*,FrameLabel->(Style[#,16,FontFamily->"Times"]& /@ {"x","\[CapitalTheta](x)"})*)
(*,Exclusions->None*)
(*,Evaluate@QpStyles]*)


(* ::Input::Initialization:: *)
QpPFScaling[x_List,p_List] := {1-#,#}&@CDF[NormalDistribution[0,Sqrt[2]],-Subtract@@(sensoryscale[#,p]& /@ x)]


(* ::Subsection::Closed:: *)
(*QpPFOrC*)


(* ::Input::Initialization:: *)
QpPFOrC[{x_},{lambda_, a_, p_, c_, q_}] :={1-#,#}&@((1+(1-2*lambda)*Erf[Piecewise[{{c*Abs[x]^q,x<=0}},a*x^p]/Sqrt[2]])/2)


(* ::Input:: *)
(*Plot[Evaluate[QpPFOrC[{x},{0, 8, .5, 8, 2}]],{x,-1,1},Evaluate@QpStyles]*)


(* ::Subsection::Closed:: *)
(*QpPFOrB*)


(* ::Input::Initialization:: *)
QpPFOrB[{strength_},{Mu_,GAMMA_,DELTA_,A_,P_,CC_,Q_}] :={1-#,#}&@Piecewise[{{DELTA+2(Mu-DELTA)CDF[NormalDistribution[0,1],-CC*Abs[strength]^Q],strength<=0},{2Mu+GAMMA-1+2(1-GAMMA-Mu)CDF[NormalDistribution[0,1],A*strength^P],strength>0}}]


(* ::Input:: *)
(*Plot[Evaluate[QpPFOrB[{x},{.7,0,0, 8, .5, 8, .5}]],{x,-1,1},Evaluate@QpStyles]*)


(* ::Subsection::Closed:: *)
(*QpLongest1*)


(* ::Text:: *)
(*From fitting artificial bias.nb*)


(* ::Input::Initialization:: *)
QpPFLongest1[{totalLengthS_,totalLengthT_,p_,t_},{mu_,sigma_,lapseRate_,leftLimit_,rightLimit_,agoal_,pgoal_}] :={1-#,#}&@Module[{(*fT,FS,SDTportion*)},FS[x_]=(Erfc[(((1-mu-p)*totalLengthS)/2-x)/(Sqrt[2]*sigma)]*Erfc[(((1+mu+p)*totalLengthS)/2-x)/(Sqrt[2]*sigma)])/4;fT[x_]=Erfc[(((1-mu-p-t)*totalLengthT)/2-x)/(Sqrt[2]*sigma)]/(2*E^((((1+mu+p+t)*totalLengthT)/2-x)^2/(2*sigma^2))*Sqrt[2*Pi]*sigma)+Erfc[(((1+mu+p+t)*totalLengthT)/2-x)/(Sqrt[2]*sigma)]/(2*E^((((1-mu-p-t)*totalLengthT)/2-x)^2/(2*sigma^2))*Sqrt[2*Pi]*sigma);
SDTportion=NIntegrate[FS[Chop[x]]fT[Chop[x]],{x,-leftLimit,rightLimit},AccuracyGoal->agoal,PrecisionGoal->pgoal];(1-lapseRate)*SDTportion+(1-SDTportion)*lapseRate]


(* ::Input:: *)
(*Plot[Evaluate[QpPFLongest1[{100,100,-.04,t},{.016,.47,.01,48,55,5,4}]],{t,-.04,.04},Evaluate@QpStyles]*)


(* ::Input:: *)
(*ListPlot[Transpose[Table[Transpose[{{t,t},QpPFLongest1[{100,100,-.04,t},{.016,.47,.01,0,70,23,12}]}],{t,-.04,.04,.00125}]],Joined->True,##]&@@QpStyles*)


(* ::Input:: *)
(*QpPFLongest1[{100,100,-.04,.02},{.016,.47,.01,0,70,23,12}]*)


(* ::Input:: *)
(*Timing[Table[QpPFLongest1[{100,100,-.04,.02},{.016,.47,.01,0,70,23,12}],{1000}]][[1]]*)


(* ::Subsection::Closed:: *)
(*QpCOCSDT*)


(* ::Text:: *)
(*From fitting artificial bias.nb*)


(* ::Input::Initialization:: *)
QpCOCSDT[{p_,t_},{mu_,sigma_,lapseRate_}] :={1-#,#}&@Module[{},lapseRate+(1-2lapseRate)(1+Erf[t/(Sqrt[2]*sigma)]*Erf[(2*(mu+p)+t)/(Sqrt[2]*sigma)])/2]


(* ::Input:: *)
(*Plot[Evaluate[QpCOCSDT[{-.04,t},{.016,.013,.01}]],{t,-.04,.04},Evaluate@QpStyles]*)


(* ::Section::Closed:: *)
(*Examples*)


(* ::Text:: *)
(*In this section we show examples that illustrate the wide variety of applications of the QuestPlus method. In all but the last example, we create a simulated observer suitable to the application. In the last case, we conduct a real experiment on the user to illustrate the steps involved.*)
(*In each case, we first configure the experiment by setting some options, and then run QuestPlus, and then analyze and plot the results.*)


(* ::Subsection::Closed:: *)
(*Threshold*)


(* ::Text:: *)
(*We estimate a contrast threshold, assuming a Weibull psychometric function. This case uses the default options for QuestPlus, which we review here.*)


(* ::Input:: *)
(*Options[QuestPlus]*)


(* ::Text:: *)
(*Note that the outcome function is defined to be a simulated observer. We run QuestPlus for 32 trials and plot the fitted result.*)


(* ::Input:: *)
(*result=QuestPlus[32]*)
(*QpPlot[result]*)


(* ::Subsection::Closed:: *)
(*Threshold, slope (and more)*)


(* ::Subsubsection::Closed:: *)
(*QpPFWeibull*)


(* ::Text:: *)
(*Here we estimate two parameters of the psychometric function, threshold and slope.*)


(* ::Input:: *)
(*options = Sequence[*)
(*QpOutcome->(SimulatedObserver[#,QpPFWeibull,{-20,3,.5,.02}]& )*)
(*,QpPNames->{"Threshold","Slope","Guess","Lapse"}*)
(*,QpXNames->{"Contrast (dB)"}*)
(*,QpXSamples->{Range[-40,0]}*)
(*,QpPSamples->{Range[-40,0],Range[2,5],{.5},{0.02}}*)
(*,QpPsi->QpPFWeibull*)
(*,QpPrior->"Constant"];*)
(*result=QuestPlus[64, options]*)
(*QpPlot[result,options]*)


(* ::Subsubsection::Closed:: *)
(*QpPFOrC*)


(* ::Input:: *)
(*options = Sequence[*)
(*QpOutcome->(SimulatedObserver[#,QpPFOrC,{0, 8, 1, 8, 1}]& )*)
(*,QpPNames->{"lambda","a","p","c","q"}*)
(*,QpXNames->{"Strength"}*)
(*,QpXSamples->{Range[-1,1,.05]}*)
(*,QpPSamples->{Range[0,.04,.01],Range[8],{1},Range[8],{1}}*)
(*,QpPsi->QpPFOrC*)
(*,QpPrior->"Constant"];*)
(*result=QuestPlus[96, options]*)
(*QpPlot[result,options]*)


(* ::Subsubsection::Closed:: *)
(*QpPFLongest1*)


(* ::Input:: *)
(*options = Sequence[*)
(*QpOutcome->(SimulatedObserver[#,QpPFLongest1,{-.02,.5,.01,0,70,23,12}]& )*)
(*,QpPNames->{"mu","sigma","lapseRate","leftLimit","rightLimit","agoal","pgoal"}*)
(*,QpXNames->{"totalLengthS","totalLengthT","pedestal","test"}*)
(*,QpXSamples->{{100},{100},Range[-.08,.08,.04],Range[-.16,.16,.04]}*)
(*,QpPSamples->{Range[-.04,.04,.01],Range[.4,1,.1],{.01},{0},{70},{23},{12}}*)
(*,QpPsi->QpPFLongest1*)
(*,QpPrior->"Constant"];*)


(* ::Input:: *)
(*SeedRandom[0];result=QuestPlus[320, options];*)


(* ::Input:: *)
(*Timing[QpFit[result, options]]*)


(* ::Subsection::Closed:: *)
(*Threshold, slope, lapse*)


(* ::Text:: *)
(*Here we estimate three parameters of the psychometric function, threshold, slope, and lapse.*)


(* ::Input:: *)
(*options = Sequence[*)
(*QpOutcome->(SimulatedObserver[#,QpPFWeibull,{-20,3,.5,.03}]& )*)
(*,QpXNames->{"Contrast (dB)"}*)
(*,QpPNames->{"Threshold (dB)","Slope","Guess","Lapse"}*)
(*,QpXSamples->{Range[-40,0]}*)
(*,QpPSamples->{Range[-40,0],Range[2,5],{.5},Range[0,.04,.01]}*)
(*,QpPsi->QpPFWeibull];*)
(*result=QuestPlus[128,options]*)
(*QpPlot[result,options]*)


(* ::Subsection::Closed:: *)
(*Mean, standard deviation, lapse*)


(* ::Text:: *)
(*Here we estimate three parameters of a cumulative Gaussian psychometric function: mean, standard deviation, and lapse. The first two parameters are given the unit of degree, as in an orientation discrimination experiment, where the observer might be asked whether a line was tilted to the left or right of vertical.*)


(* ::Input:: *)
(*options = Sequence[*)
(*QpOutcome->(SimulatedObserver[#,QpPFNormal,{1,3,.02}]& )*)
(*,QpXNames->{"Orientation (\[Degree])"}*)
(*,QpPNames->{"Mean (\[Degree])","SD (\[Degree])","Lapse"}*)
(*,QpXSamples->{Range[-10,10]}*)
(*,QpPSamples->{Range[-5,5],Range[1,10],Range[0,.04,.01]}*)
(*,QpPsi->QpPFNormal];*)
(*result = QuestPlus[128,options]*)
(*QpPlot[result,options]*)


(* ::Subsection::Closed:: *)
(*Spatial contrast sensitivity function*)


(* ::Text:: *)
(*For this case, we first define a psychometric function that responds to two stimulus variables: frequency and contrast. It has parameters of t, c0, and cf. Sensitivity in dB is a linear function of frequency, with slope cf and asymptote c0, but with sensitivity capped at t.*)


(* ::Subsubsection::Closed:: *)
(*QpPFSCSF*)


(* ::Input::Initialization:: *)
QpPFSCSF[{f_,c_},{t_,c0_,cf_}]:= QpPFWeibull[{c},{Max[t,c0+cf f],3.,.5,.01}]


(* ::Input:: *)
(*Plot3D[QpPFSCSF[{f,c},{-35,-50,1.2}][[2]],{c,-40,0},{f,0,40}]*)


(* ::Text:: *)
(*Here we plot the underlying contrast threshold function and the psychometric function, as an image.*)


(* ::Input:: *)
(*c0=-50;*)
(*cf=1.2;*)
(*t = -40;*)
(*f1=Plot[Max[t,c0+cf f],{f,0,50}*)
(*,FrameLabel->{"Spatial frequency (c/d)","Contrast threshold (dB)"}*)
(*,AspectRatio->1*)
(*,PlotRange->{{0,50},{-50,0}}*)
(*,Evaluate@QpStyles];*)
(*image=Table[QpPFSCSF[{f,c},{t,c0 ,cf}][[2]],{c,-50,0},{f,0,50}];*)
(*f2=psifig=ListDensityPlot[image*)
(*,ColorFunction->GrayLevel*)
(*,ColorFunctionScaling->False*)
(*,DataRange->{{0,50},{-50,0}}*)
(*,FrameLabel->{"Spatial Frequency (c/d)","Contrast (dB)"}*)
(*,QpStyles];*)
(*Row[{"A",Spacer[10],f1,Spacer[10],"B",Spacer[10],f2},Alignment->{Center,Bottom},BaseStyle->24]*)


(* ::Subsubsection::Closed:: *)
(*Result*)


(* ::Text:: *)
(*We configure the experiment by setting the options, defining a simulated observer with our new psychometric function.*)


(* ::Input:: *)
(*options=Sequence[*)
(*QpOutcome-> (SimulatedObserver[#,QpPFSCSF,{-35,-50,1.2}]& )*)
(*,QpPNames->{"t","\!\(\*SubscriptBox[\(c\), \(0\)]\)","\!\(\*SubscriptBox[\(c\), \(f\)]\)"}*)
(*,QpXNames->{"Spatial frequency (c/d)","Contrast (dB)"}*)
(*,QpXSamples->{Range[0,40,2],Range[-50,0,2]}*)
(*,QpPSamples->{Range[-30,-50,-2],Range[-40,-60,-2],Range[.8,1.6,.2]}*)
(*,QpPsi->QpPFSCSF];*)


(* ::Text:: *)
(*And we run the experiment.*)


(* ::Input:: *)
(*result=QuestPlus[32,options]*)


(* ::Input:: *)
(*result = {{-32,-56,1.4000000000000001`},{{{40,0},2},{{40,-4},1},{{34,0},2},{{36,0},2},{{0,-38},2},{{38,0},2},{{0,-40},1},{{38,0},2},{{18,-26},2},{{18,-26},2},{{40,0},2},{{0,-36},2},{{0,-36},1},{{40,0},2},{{20,-26},2},{{20,-26},2},{{40,-2},2},{{22,-26},2},{{40,-6},1},{{20,-26},1},{{40,0},2},{{18,-26},2},{{40,0},2},{{18,-26},2},{{0,-32},2},{{0,-32},2},{{0,-34},2},{{0,-34},1},{{40,0},1},{{18,-26},2},{{38,0},2},{{18,-26},2}}};*)


(* ::Input:: *)
(*QpPlot2D[result,options]*)


(* ::Input:: *)
(**)


(* ::Input:: *)
(*ListPlot3D[Reverse[Transpose[posterior[[All,3,All]]]],PlotRange->All*)
(*,DataRange->(MinMax /@(QpPSamples /. {options}))[[{1,3}]]*)
(*,AxesLabel->{"\!\(\**)
(*StyleBox[\"a\",\nFontSlant->\"Italic\"]\)","  \n\!\(\**)
(*StyleBox[\"t\",\nFontSlant->\"Italic\"]\)",Rotate["Probability\n",Pi/2]}*)
(*,Mesh->Full*)
(*,BaseStyle->{12,FontFamily->"Helvetica",AbsolutePointSize[6]}*)
(*,ImageSize->{Automatic,240}]*)


(* ::Input:: *)
(*result=QpRun[32,options]*)


(* ::Input:: *)
(*QpPlot2D[result,options]*)


(* ::Subsection::Closed:: *)
(*Spatiotemporal contrast sensitivity function*)


(* ::Text:: *)
(*Here we measure the spatio-temporal contrast sensitivity function. We start by defining an appropriate psychometric  function.*)


(* ::Subsubsection::Closed:: *)
(*QpPFSTCSF*)


(* ::Text:: *)
(*This function assumes threshold in dB is a linear  function of spatial and temporal contrast sensitivity (Watson and Ahumada, 2016), with asymptote c0, and slopes cf and cw, with minimum threshold at t.*)


(* ::Input:: *)
(*Clear[QpPFSTCSF]*)


(* ::Input::Initialization:: *)
QpPFSTCSF[{c_,f_, w_},{t_,c0_,cf_,cw_}]:= QpPFWeibull[{c},{Max[t,c0+cf f+cw w],3.,.5,.01}]


(* ::Input:: *)
(*c0=-50;*)
(*cf=1.2;*)
(*cw=1.;*)
(*t = -35;*)
(*tst=Plot3D[Max[t,c0+cf f+cw w],{f,0,40},{w,0,40}*)
(*,AxesLabel->{"Spatial frequency \n(c/d)","\nTemporal Frequency \n(Hz)","\nContrast threshold \n(dB)"}*)
(*,AxesEdge->{Automatic,{1,-1},Automatic}*)
(*,BaseStyle->{12,FontFamily->"Helvetica",AbsolutePointSize[6]}*)
(*,PlotRange->{-40,0}*)
(*,BoxRatios->{1, 1, .6}*)
(*,Exclusions->None*)
(*,ImageSize->{Automatic,240}]*)


(* ::Input:: *)
(*image=Table[QpPFSTCSF[{-30,f, w},{t,c0,cf,cw}][[2]],{w,0,50},{f,0,50}];*)


(* ::Input:: *)
(*psifig=ListDensityPlot[image*)
(*,ColorFunction->GrayLevel*)
(*,ColorFunctionScaling->False*)
(*,DataRange->{{0,50},{-50,0}}*)
(*,FrameLabel->{"Spatial Frequency (c/d)","Contrast (dB)"}*)
(*,QpStyles]*)


(* ::Subsubsection::Closed:: *)
(*Result*)


(* ::Text:: *)
(*We configure the experiment by defining the options, including a simulated observer using the new psychometric function.*)


(* ::Input:: *)
(*options=Sequence[*)
(*QpOutcome-> (SimulatedObserver[#,QpPFSTCSF,{-35,-50,1.2,1.0}]& )*)
(*,QpPNames->{"t","\!\(\*SubscriptBox[\(c\), \(0\)]\)","\!\(\*SubscriptBox[\(c\), \(f\)]\)","\!\(\*SubscriptBox[\(c\), \(w\)]\)"}*)
(*,QpXNames->{"Contrast (dB)","Spatial frequency (c/d)","Temporal frequency (Hz)"}*)
(*,QpXSamples->{Range[-40,0,5],Range[0,40,5],Range[0,40,5]}*)
(*,QpPSamples->{Range[-30,-50,-5],Range[-40,-60,-5],Range[.8,1.6,.2],Range[.8,1.6,.2]}*)
(*,QpPsi->QpPFSTCSF];*)


(* ::Input:: *)
(*result=QuestPlus[256,options]*)


(* ::Input:: *)
(*QpPlot3D[result,options]*)


(* ::Subsection::Closed:: *)
(*Thurstone scaling*)


(* ::Input:: *)
(*options=Sequence[*)
(*QpOutcome-> (SimulatedObserver[#,QpPFScaling,{6,.2,.4}]& )*)
(*,QpXNames->{"\!\(\*SubscriptBox[\(x\), \(1\)]\)","\!\(\*SubscriptBox[\(x\), \(2\)]\)"}*)
(*,QpPNames->{"max","threshold","power"}*)
(*,QpXSamples->{Range[0,1,.1],Range[0,1,.1]}*)
(*,QpPSamples->{Range[1,10],Range[0,.9,.1],Range[.1,1,.1]}*)
(*,QpPsi->QpPFScaling];*)


(* ::Input:: *)
(*result = QuestPlus[128,options]*)


(* ::Input:: *)
(*QpPlot2D[result,QpPointSize->.06,options]*)


(* ::Subsection::Closed:: *)
(*Increment threshold*)


(* ::Text:: *)
(*We consider a blur discrimination experiment, but the increment could be of an stimulus attribute.*)


(* ::Text:: *)
(*We define a model (see Watson and Ahumada, 2011) for the blur increment, as a function of the standard deviation sigma of the reference blur (the pedestal) and two parameters. *)


(* ::Input::Initialization:: *)
blurinc[sigma_,omega_,beta_] := -sigma+Sqrt[beta^2(omega^2-1)+sigma^2omega^2]


(* ::Text:: *)
(*Here is an example.*)


(* ::Input:: *)
(*omega=1.2;*)
(*beta=1;*)
(*Plot[Log[10,blurinc[10^lr,omega,beta]]*)
(*,{lr,-1.5,1.5}*)
(*,PlotRange->{-1,1}*)
(*,AspectRatio->Automatic*)
(*,FrameLabel->{"Reference blur (log arcmin)","Increment blur (log arcmin)"}*)
(*,Evaluate@QpStyles]*)


(* ::Text:: *)
(*Now we define the psychometric function. It is a Weibull function of distance from the increment threshold.*)


(* ::Input:: *)
(*Clear[QpPFBlur]*)


(* ::Input::Initialization:: *)
QpPFBlur[{lsigma_,ldelta_},{omega_,beta_}] :=QpPFWeibull[{ldelta},{Log[10,blurinc[10^lsigma,omega,beta]],40,.5,.01}]


(* ::Text:: *)
(*Here we picture the psychometric function as an image.*)


(* ::Input:: *)
(*psi=QpPFBlur;*)
(*parameters={1.2,1};*)
(*xsamples= {Range[-1,1.5,.25],Range[-1,1,.25]};*)
(*Length /@ xsamples*)
(*ranges=MinMax /@ xsamples;*)
(*image= Table[psi[{x,y},parameters][[2]],{y,xsamples[[2]]},{x,xsamples[[1]]}];*)
(*psifig=ArrayPlot[Reverse[image]*)
(*,ColorFunction->GrayLevel*)
(*,ColorFunctionScaling->False*)
(*,DataRange->ranges*)
(*,QpStyles]*)


(* ::Text:: *)
(*We configure the experiment.*)


(* ::Input:: *)
(*options=Sequence[*)
(*QpOutcome-> (SimulatedObserver[#,QpPFBlur,{1.2,1}]& )*)
(*,QpPNames->{"omega","beta"}*)
(*,QpXNames->{"Reference blur (log arcmin)","Threshold blur (log arcmin)"}*)
(*,QpXSamples->{Range[-1,1.5,.25],Range[-1,1,.25]}*)
(*,QpPSamples->{Range[1.05,1.5,.05],Range[.5,2,.1]}*)
(*,QpPsi->QpPFBlur];*)


(* ::Input:: *)
(*result = QuestPlus[64,options]*)


(* ::Text:: *)
(*We plot the result.*)


(* ::Input:: *)
(*QpPlot2D[result,options]*)


(* ::Subsection::Closed:: *)
(*Categorical rating*)


(* ::Subsubsection::Closed:: *)
(*3 categories*)


(* ::Input:: *)
(*options= Sequence[QpOutcome->(SimulatedObserver[#,QpPFRating,{.25,1,1}]& )*)
(*,QpXNames->{"Magnitude"}*)
(*,QpPNames->{"SD","Width1","Width2"}*)
(*,QpXSamples->{Range[0,3,.25]}*)
(*,QpPSamples->{Range[.05,.5,.05],Range[.5,2,.5],Range[.5,2,.5]}*)
(*,QpPsi->QpPFRating];*)


(* ::Input:: *)
(*result=QuestPlus[64,options]*)


(* ::Input:: *)
(*QpPlotCategories[result,options]*)


(* ::Subsubsection::Closed:: *)
(*4 categories*)


(* ::Input:: *)
(*options= Sequence[QpOutcome->(SimulatedObserver[#,QpPFRating,{.5,1,2,3}]& )*)
(*,QpXNames->{"Magnitude"}*)
(*,QpPNames->{"SD","Width1","Width2","Width3"}*)
(*,QpXSamples->{Range[0,8,.25]}*)
(*,QpPSamples->{Range[.05,1,.05],Range[.5,4,.5],Range[.5,4,.5],Range[.5,4,.5]}*)
(*,QpPsi->QpPFRating];*)


(* ::Input:: *)
(*result=QuestPlus[128,options]*)


(* ::Input:: *)
(*QpPlotCategories[result,options]*)


(* ::Subsection::Closed:: *)
(*Circular categorization*)


(* ::Input:: *)
(*options = Sequence[*)
(*QpOutcome->(SimulatedObserver[#,QpPFCircular,{4.,Pi/3.,2Pi/3.,2Pi/3.}]& )*)
(*,QpPNames->{"Concentration","c1","w1","w2"}*)
(*,QpXSamples->{Range[0,17]Pi/9.}*)
(*,QpXNames->{"Direction"}*)
(*,QpPSamples->{Range[1,8],Range[0,17]Pi/9.,{2Pi/3.},{2Pi/3.}}*)
(*,QpPsi->QpPFCircular];*)


(* ::Input:: *)
(*result=QuestPlus[64,options]*)


(* ::Text:: *)
(*This part may be slow because the psychometric function involves evaluation of integrals.*)


(* ::Input:: *)
(*QpPlotCategories[result,options]*)


(* ::Subsection::Closed:: *)
(*A Real Experiment*)


(* ::Text:: *)
(*In this example we construct a simple psychophysical experiment using the built-in display capabilities of Mathematica. Display properties are not calibrated, but the example is otherwise realistic.*)


(* ::Text:: *)
(*The experiment is letter identification as a function of contrast in the presence of noise. First we define a few parameters (which the reader can modify), and create some letter stimuli, and a noise stimulus.*)


(* ::Input:: *)
(*imagesize=64;*)
(*fontsize=32;*)
(*noisecontrast=.25;*)
(*magnification=2;*)
(*duration=0.25;*)
(*SloanList = Characters["CDHKNORSVZ"];*)


(* ::Text:: *)
(*Now we create our stimuli.*)


(* ::Input:: *)
(*letters = ColorNegate[Image[Graphics[Text[Style[#,FontFamily->"Sloan",FontSize->fontsize]],ImageSize->imagesize],ColorSpace->"Grayscale"]]& /@ SloanList*)
(**)


(* ::Text:: *)
(*Now we create the noise image.*)


(* ::Input:: *)
(*noise = Image@RandomReal[{0,1},imagesize{1,1}]*)


(* ::Text:: *)
(*Next we create a function to create a stimulus consisting of the addition of a background of 1/2, a letter of some contrast, and noise of some contrast. Index selects one of the list of Sloan letters.*)


(* ::Input:: *)
(*Stimulus[index_, lettercontrast_,noisecontrast_,magnification_:2] := Image[ImageAdd[ImageMultiply[letters[[index]],lettercontrast .5],ImageMultiply[ImageAdd[noise,-.5],noisecontrast],.5],Magnification ->magnification]*)


(* ::Text:: *)
(*We test this function.*)


(* ::Input:: *)
(*Stimulus[3, .25,noisecontrast,magnification]*)


(* ::Text:: *)
(*Now we create a blank stimulus, in which the letter contrast is zero. This will be used between trials.*)


(* ::Input:: *)
(*blank=Stimulus[1, 0,noisecontrast,magnification]*)


(* ::Text:: *)
(*Next we create the function that is called by QuestPlus whenever a trail is conducted. We include creation of the stimulus, playing an alert tone, presenting the stimulus, replacing it with a blank after a specified duration, collecting the response from the subject, providing feedback, and finally returning the result (1 for incorrect,  2 for correct).*)


(* ::Input:: *)
(*Clear[ExampleResponse]*)


(* ::Input:: *)
(*ExampleResponse[strength_] :=Block[*)
(*{response,index,stimulus,correct},*)
(*index = RandomInteger[{1,10}];*)
(*stimulus=Stimulus[index, dBInverse[strength],noisecontrast,magnification] ;*)
(*EmitSound[Sound[SoundNote[0,.25]]];*)
(*CreatePalette[stimulus,window,WindowMargins->Automatic,WindowFrame->"Frameless",WindowTitle->None,WindowFloating->True];*)
(*Pause[duration];*)
(*CreatePalette[blank,window,WindowMargins->Automatic,WindowFrame->"Frameless",WindowTitle->None,WindowFloating->True];*)
(*response=ChoiceDialog["Choose:",SloanList,WindowSize->{800,100},WindowMargins->{{Automatic,Automatic},{0,Automatic}}];correct=(response==SloanList[[index]]);*)
(*Speak[If[correct, "right","wrong"]];*)
(*Pause[1];*)
(*If[correct, 2,1]*)
(*]*)


(* ::Text:: *)
(*Before we can try this out, we need to create the window within which the stimuli will appear. We fill it initially with the noise image at the specified contrast. By default, this creates a floating window in the center of the screen.*)


(* ::Input:: *)
(*window=CreatePalette[blank,WindowMargins->Automatic,WindowFrame->"Frameless",WindowTitle->None,WindowFloating->True];*)


(* ::Text:: *)
(*Now we can try the ExampleResponse function. You should hear a tone, see a letter, and see a response panel appear near the bottom of the screen. Select your response. You should hear speech feedback.*)


(* ::Input:: *)
(*ExampleResponse[-10]*)


(* ::Text:: *)
(*Now we configure QuestPlus by defining values for some options.*)


(* ::Input:: *)
(*nstimuli=Length[letters];*)
(*QpOptions = Sequence[*)
(*QpOutcome->(ExampleResponse[#[[1]]]& )*)
(*,QpXNames->{"Contrast (dB)"}*)
(*,QpPNames->{"Contrast (dB)","Slope","Guess","Lapse"}*)
(*,QpXSamples->{Range[-40,0,2]}*)
(*,QpPSamples->{Range[-40,0,1],{3},{1./nstimuli},{.01}}*)
(*,QpPsi->QpPFWeibull];*)


(* ::Text:: *)
(*Now we are ready to conduct the experiment. We guess that threshold will be at -20 dB, and we decide to collect 16 trials. When you are ready, evaluate the following expression, and respond to each trial.*)


(* ::Input:: *)
(*result = QuestPlus[16,QpOptions]*)


(* ::Text:: *)
(*When we are done, we close the stimulus window.*)


(* ::Input:: *)
(*NotebookClose[window];*)


(* ::Text:: *)
(*Now we can fit the psychometric function to the data and plot the result.*)


(* ::Input:: *)
(*QpPlot[result,QpOptions]*)
(**)


(* ::Section::Closed:: *)
(*Demonstration*)


(* ::Text:: *)
(*This is the code to create an interactive demonstration of all of the examples shown in this Notebook.*)


(* ::Input:: *)
(*dbase = {*)
(*{"Threshold",{},QpPlot}*)
(*,*)
(*{"Threshold, slope",{QpOutcome->(SimulatedObserver[#,QpPFWeibull,{-20,3,.5,.02}]& )*)
(*,QpPNames->{"Threshold","Slope","Guess","Lapse"}*)
(*,QpXNames->{"Contrast (dB)"}*)
(*,QpXSamples->{Range[-40,0]}*)
(*,QpPSamples->{Range[-40,0],Range[2,5],{.5},{0.02}}*)
(*,QpPsi->QpPFWeibull*)
(*,QpPrior->"Constant"},QpPlot}*)
(*,*)
(*{"Threshold, slope, lapse",{QpOutcome->(SimulatedObserver[#,QpPFWeibull,{-20,3,.5,.03}]& )*)
(*,QpXNames->{"Contrast (dB)"}*)
(*,QpPNames->{"Threshold (dB)","Slope","Guess","Lapse"}*)
(*,QpXSamples->{Range[-40,0]}*)
(*,QpPSamples->{Range[-40,0],Range[2,5],{.5},Range[0,.04,.01]}*)
(*,QpPsi->QpPFWeibull},QpPlot}*)
(*,*)
(*{"Mean, standard deviation, lapse",{QpOutcome->(SimulatedObserver[#,QpPFNormal,{1,3,.02}]& )*)
(*,QpXNames->{"Orientation (\[Degree])"}*)
(*,QpPNames->{"Mean (\[Degree])","SD (\[Degree])","Lapse"}*)
(*,QpXSamples->{Range[-10,10]}*)
(*,QpPSamples->{Range[-5,5],Range[1,10],Range[0,.04,.01]}*)
(*,QpPsi->QpPFNormal},QpPlot}*)
(*,*)
(*{"Spatial contrast sensitivity function",{QpOutcome-> (SimulatedObserver[#,QpPFSCSF,{-35,-50,1.2}]& )*)
(*,QpPNames->{"t","\!\(\*SubscriptBox[\(c\), \(0\)]\)","\!\(\*SubscriptBox[\(c\), \(f\)]\)"}*)
(*,QpXNames->{"Spatial frequency (c/d)","Contrast (dB)"}*)
(*,QpXSamples->{Range[0,40,2],Range[-50,0,2]}*)
(*,QpPSamples->{Range[-30,-50,-2],Range[-40,-60,-2],Range[.8,1.6,.2]}*)
(*,QpPsi->QpPFSCSF},QpPlot2D}*)
(*,*)
(*{"Spatiotemporal contrast sensitivity function",{QpOutcome-> (SimulatedObserver[#,QpPFSTCSF,{-35,-50,1.2,1.0}]& )*)
(*,QpPNames->{"t","\!\(\*SubscriptBox[\(c\), \(0\)]\)","\!\(\*SubscriptBox[\(c\), \(f\)]\)","\!\(\*SubscriptBox[\(c\), \(w\)]\)"}*)
(*,QpXNames->{"Contrast (dB)","Spatial frequency (c/d)","Temporal frequency (Hz)"}*)
(*,QpXSamples->{Range[-40,0,5],Range[0,40,5],Range[0,40,5]}*)
(*,QpPSamples->{Range[-30,-50,-5],Range[-40,-60,-5],Range[.8,1.6,.2],Range[.8,1.6,.2]}*)
(*,QpPsi->QpPFSTCSF}, QpPlot3D}*)
(*,{"Thurstone scaling",{QpOutcome-> (SimulatedObserver[#,QpPFScaling,{6,.2,.4}]& )*)
(*,QpXNames->{"\!\(\*SubscriptBox[\(x\), \(1\)]\)","\!\(\*SubscriptBox[\(x\), \(2\)]\)"}*)
(*,QpPNames->{"max","threshold","power"}*)
(*,QpXSamples->{Range[0,1,.1],Range[0,1,.1]}*)
(*,QpPSamples->{Range[1,10],Range[0,.9,.1],Range[.1,1,.1]}*)
(*,QpPsi->QpPFScaling},QpPlot2D}*)
(*,*)
(*{"Increment threshold",{QpOutcome-> (SimulatedObserver[#,QpPFBlur,{1.2,1}]& )*)
(*,QpPNames->{"omega","beta"}*)
(*,QpXNames->{"Reference blur (log arcmin)","Threshold blur (log arcmin)"}*)
(*,QpXSamples->{Range[-1,1.5,.25],Range[-1,1,.25]}*)
(*,QpPSamples->{Range[1.05,1.5,.05],Range[.5,2,.1]}*)
(*,QpPsi->QpPFBlur},QpPlot2D}*)
(*,{"Categorical rating",{QpOutcome->(SimulatedObserver[#,QpPFRating,{.25,1,1}]& )*)
(*,QpXNames->{"Magnitude"}*)
(*,QpPNames->{"SD","Width1","Width2"}*)
(*,QpXSamples->{Range[0,3,.25]}*)
(*,QpPSamples->{Range[.05,.5,.05],Range[.5,2,.5],Range[.5,2,.5]}*)
(*,QpPsi->QpPFRating},QpPlotCategories}*)
(*,*)
(*{"Circular categorization",{QpOutcome->(SimulatedObserver[#,QpPFCircular,{4,Pi/3,2Pi/3,2Pi/3}]& )*)
(*,QpPNames->{"Concentration","c1","w1","w2"}*)
(*,QpXSamples->{Range[0,11]Pi/6}*)
(*,QpXNames->{"Direction"}*)
(*,QpPSamples->{Range[1,8],Range[0,11]Pi/6,{2Pi/3},{2Pi/3}}*)
(*,QpPsi->QpPFCircular},QpPlotCategories}*)
(*};*)
(**)


(* ::Input:: *)
(*ncases = Length[dbase];*)
(*demo = Labeled[Manipulate[( *)
(*options = Sequence@@dbase[[case,2]];*)
(*result = QuestPlus[ntrials, options];*)
(*fig=dbase[[case,3]][result,options];*)
(*{names,outcomefunction,psi} ={QpPNames,QpOutcome,QpPsi} /.dbase[[case,2]]/. Options[QuestPlus];*)
(*Column[{Style[dbase[[case,1]],24]*)
(*,Framed[Row[{Style["Simulated Observer",14],*)
(*,Grid[Transpose[{names,outcomefunction[[1,3]]}],BaseStyle->12]}],FrameStyle->GrayLevel[.8]]*)
(*,fig},Alignment->Center]*)
(*)*)
(*,{{ntrials,32,"Trials"},{16,32,64,128,256},LabelStyle->DemoControlLabelStyle,ControlType->PopupMenu}*)
(*,{{case,1,"Case"},Range[ncases],LabelStyle->DemoControlLabelStyle}*)
(**)
(*,ContinuousAction->False*)
(*,TrackedSymbols:>{ntrials,case}*)
(*,Evaluate[Sequence @@DemoStyle]*)
(*,FrameLabel->DemoFrameLabels["QUEST+ Demonstration","\nThis demonstration illustrates the performance of the QUEST+ generalized Bayesian adaptive testing procedure.","Instructions: The user can select one of ten cases. Each is a particular experimental scenario, indicated by the label above the figure. The parameters of the simulated observer are shown. The figure shows the outcome of the experiment, along with estimated parameters and psychometric function.","","References: *)
(*Watson, A. B. (2017). QUEST+: A general multidimensional Bayesian adaptive psychometric method. Journal of Vision."]]*)
(*,Style["\[Copyright] Andrew B. Watson. Version: "<>DateTimeString,10,Gray]]*)


(* ::Input:: *)
(*Export["QuestPlusDemo.cdf",demo]*)


(* ::Subsection::Closed:: *)
(*Creating animation*)


(* ::Text:: *)
(*This is an animation used in the journal demonstration page.*)


(* ::Input:: *)
(*casefig[case_,ntrials_] := *)
(*( *)
(*options = Sequence@@dbase[[case,2]];*)
(*result = QuestPlus[ntrials, options];*)
(*fig=dbase[[case,3]][result,options];*)
(*{names,outcomefunction,psi} ={QpPNames,QpOutcome,QpPsi} /.dbase[[case,2]]/. Options[QuestPlus];*)
(*Labeled[*)
(*Column[{Style[dbase[[case,1]],24]*)
(*,Framed[Row[{Style["Simulated Observer",14],*)
(*,Grid[Transpose[{names,outcomefunction[[1,3]]}],BaseStyle->12]}],FrameStyle->GrayLevel[.8]]*)
(*,fig},Alignment->Center,ItemSize->{60,Automatic}]*)
(*,Style["QUEST+ Demonstration\n",DemoTitleStyle],Top*)
(*,LabelStyle->{18,FontColor->GrayLevel[0.5],FontFamily->"Helvetica-Neue"}*)
(*,BaseStyle->{14,FontFamily->"Helvetica"}*)
(*,Alignment->Center*)
(*,ImageSize->{Automatic,380}]*)
(*)*)


(* ::Input:: *)
(*figs=MapThread[casefig[#1,#2]& ,{ {1,2,3,4,5,7,8,9,10},{32,64,64,64,64,64,64,64,64}}]*)


(* ::Input:: *)
(*Export["QuestPlusDemo.gif",figs,"DisplayDurations"->1]*)
